@@ -39,7 +39,7 @@ def _unfoldMatrix(a):
     return m
 
 
-def computeGradient(im, mode="gaussian", mask=None):
+def computeGradient(im, mode="gaussian", mask=None, anisotropy=numpy.ones(3)):
     """
     Returns the gradient of passed greylevel image.
 
@@ -52,6 +52,10 @@ def computeGradient(im, mode="gaussian", mask=None):
             Selects method to compute gradients, can be either "splines", "gaussian" or "finite_difference".
             Default is "gaussian"
 
+        anisotropy : array_like
+            Relative pixel size for all axis. If your z-step is e.g. 2 times bigger than the pixel size in
+            x and y, this parameter should be set to [2, 1, 1].
+
     Returns
     --------
         gradients : tuple of arrays
@@ -89,6 +93,9 @@ def computeGradient(im, mode="gaussian", mask=None):
         elif mode == "finite_difference":
             Gz, Gy, Gx = numpy.gradient(im)
 
+        Gz = Gz / anisotropy[0]
+        Gy = Gy / anisotropy[1]
+        Gx = Gx / anisotropy[2]
         return (Gz, Gy, Gx)
 
     else:
@@ -107,6 +114,8 @@ def computeGradient(im, mode="gaussian", mask=None):
         elif mode == "finite_difference":
             Gy, Gx = numpy.gradient(im)
 
+        Gy = Gy / anisotropy[0]
+        Gx = Gx / anisotropy[1]
         return (Gy, Gx)
 
 
@@ -163,29 +172,29 @@ def computeStructureTensor(gradients, sigma, mask=None):
     return structureTensor
 
 
-def computeGradientStructureTensor(im, sigma, mode="gaussian"):
+def computeGradientStructureTensor(im, sigma, mode="gaussian", anisotropy=numpy.ones(3)):
     """
-    This function calls `computeGradient` with the mode passed in `mode` and then computes the structure tensor for each pixel (integrating in a Gaussian window of size `sigma`) with `computeStructureTensor` and returns that directly as a 3 x N x M or a 6 x N x M x O array.
+    This function calls `computeGradient` with the mode and anisotropy factors passed in `mode` and `anisotropy_factors` respectively and then computes the structure tensor for each pixel (integrating in a Gaussian window of size `sigma`) with `computeStructureTensor` and returns that directly as a 3 x N x M or a 6 x N x M x O array.
     """
-    print("Computing gradients...", end="")
-    g = computeGradient(im, mode=mode)
-    print("done")
-    print("Computing structure tensor...", end="")
+    # print("Computing gradients...", end="")
+    g = computeGradient(im, mode=mode, anisotropy=anisotropy)
+    # print("done")
+    # print("Computing structure tensor...", end="")
     st = computeStructureTensor(g, sigma)
-    print("done")
+    # print("done")
     return st
 
 
-def computeGradientStructureTensorBoxes(im, boxSize, mode="gaussian"):
+def computeGradientStructureTensorBoxes(im, boxSize, mode="gaussian", anisotropy=numpy.ones(3)):
     """
-    This function calls `computeGradient` with the mode passed in `mode` and then computes the structure tensor in touching 2/3D boxes with `computeStructureTensorBoxes` and returns the structure tensor for each box as a flattened 3 x N x M or a 6 x N x M x O array
+    This function calls `computeGradient` with the mode and anisotropy factors passed in `mode` and `anisotropy_factors` respectively and then computes the structure tensor in touching 2/3D boxes with `computeStructureTensorBoxes` and returns the structure tensor for each box as a flattened 3 x N x M or a 6 x N x M x O array
     """
-    print("Computing gradients...", end="")
-    g = computeGradient(im, mode=mode)
-    print("done")
-    print("Computing structure tensor...", end="")
+    # print("Computing gradients...", end="")
+    g = computeGradient(im, mode=mode, anisotropy=anisotropy)
+    # print("done")
+    # print("Computing structure tensor...", end="")
     st = computeStructureTensorBoxes(g, boxSize)
-    print("done")
+    # print("done")
     return st
 
 
@@ -305,7 +314,7 @@ def computeStructureTensorBoxes(gradients, boxSize, mask=None, returnBoxCenters=
         print(f"computeStructureTensorBoxes(): Unknown number of gradient dimensions: {len(gradients)}")
 
 
-@numba.njit(parallel=True)
+@numba.njit(parallel=True, cache=True)
 def orientationFunction(structureTensor, progressProxy, fibre=True, computeEnergy=False, computeCoherency=False):
     symmetricComponents3d = [[0, 0], [0, 1], [0, 2], [1, 1], [1, 2], [2, 2]]
 
@@ -315,6 +324,7 @@ def orientationFunction(structureTensor, progressProxy, fibre=True, computeEnerg
     energy = numpy.zeros(structureTensor.shape[1:], dtype="<f4")
     coherency = numpy.zeros(structureTensor.shape[1:], dtype="<f4")
 
+    # It's a pity this doesn't work because we're allocating Energy and Coherency even if we don't want them
     # if computeEnergy:
     # energy = numpy.zeros(structureTensor.shape[1:], dtype=float)
     # else:
